@@ -28,7 +28,7 @@ def is_dir_usable(pth):
 		os.mknod(pthj(pth,".test"))
 		os.remove(pthj(pth,".test"))
 		return True
-	except:
+	except Exception:
 		return False
 
 def get_env_vars(key,pathsuffix=[]):
@@ -149,7 +149,7 @@ malojaconfig = Configuration(
 			"cache_expire_positive":(tp.Integer(),								"Image Cache Expiration", 				60,		"Days until images are refetched"),
 			"cache_expire_negative":(tp.Integer(),								"Image Cache Negative Expiration",		5,		"Days until failed image fetches are reattempted"),
 			"db_max_memory":(tp.Integer(min=0,max=100),							"RAM Percentage soft limit",			80,		"RAM Usage in percent at which Maloja should no longer increase its database cache."),
-			"use_request_cache":(tp.Boolean(),									"Use request-local DB Cache",			True),
+			"use_request_cache":(tp.Boolean(),									"Use request-local DB Cache",			False),
 			"use_global_cache":(tp.Boolean(),									"Use global DB Cache",					True)
 		},
 		"Fluff":{
