@@ -320,7 +320,8 @@ class MTRangeComposite(MTRangeGeneric):
 		if self.since is None: return FIRST_SCROBBLE
 		else: return self.since.first_stamp()
 	def last_stamp(self):
-		if self.to is None: return int(datetime.utcnow().replace(tzinfo=timezone.utc).timestamp())
+		#if self.to is None: return int(datetime.utcnow().replace(tzinfo=timezone.utc).timestamp())
+		if self.to is None: return today().last_stamp()
 		else: return self.to.last_stamp()
 
 	def next(self,step=1):
@@ -430,7 +431,7 @@ def time_fix(t):
 	try:
 		t = [int(p) for p in t]
 		return MTRangeGregorian(t[:3])
-	except:
+	except Exception:
 		pass
 
 	if isinstance(t[1],str) and t[1].startswith("w"):
@@ -438,7 +439,7 @@ def time_fix(t):
 			year = int(t[0])
 			weeknum = int(t[1][1:])
 			return MTRangeWeek(year=year,week=weeknum)
-		except:
+		except Exception:
 			raise
 
 
