@@ -117,10 +117,15 @@ def import_scrobbles(inputf):
 	return result
 
 def parse_spotify_lite(inputf):
-	inputfolder = os.path.dirname(inputf)
+	pth = os.path
+	inputfolder = pth.relpath(pth.dirname(pth.abspath(inputf)))
 	filenames = re.compile(r'StreamingHistory[0-9]+\.json')
 	inputfiles = [os.path.join(inputfolder,f) for f in os.listdir(inputfolder) if filenames.match(f)]
 
+	if len(inputfiles) == 0:
+		print("No files found!")
+		return
+
 	if inputfiles != [inputf]:
 		print("Spotify files should all be imported together to identify duplicates across the whole dataset.")
 		if not ask("Import " + ", ".join(col['yellow'](i) for i in inputfiles) + "?",default=True):
@@ -161,11 +166,15 @@ def parse_spotify_lite(inputf):
 
 
 def parse_spotify_full(inputf):
-
-	inputfolder = os.path.dirname(inputf)
+	pth = os.path
+	inputfolder = pth.relpath(pth.dirname(pth.abspath(inputf)))
 	filenames = re.compile(r'endsong_[0-9]+\.json')
 	inputfiles = [os.path.join(inputfolder,f) for f in os.listdir(inputfolder) if filenames.match(f)]
 
+	if len(inputfiles) == 0:
+		print("No files found!")
+		return
+		
 	if inputfiles != [inputf]:
 		print("Spotify files should all be imported together to identify duplicates across the whole dataset.")
 		if not ask("Import " + ", ".join(col['yellow'](i) for i in inputfiles) + "?",default=True):
